@@ -21,38 +21,42 @@ from exponent.commands.utils import ConnectionTracker
 from exponent.core.config import is_editable_install
 from exponent.core.remote_execution import files, system_context
 from exponent.core.remote_execution.cli_rpc_types import (
+    BashToolInput,
     CliRpcRequest,
     CliRpcResponse,
     ErrorResponse,
+    GetAllFilesRequest,
+    GetAllFilesResponse,
+    TerminateRequest,
+    TerminateResponse,
     ToolExecutionRequest,
     ToolExecutionResponse,
 )
 from exponent.core.remote_execution.code_execution import (
     execute_code_streaming,
 )
+from exponent.core.remote_execution.files import file_walk
 from exponent.core.remote_execution.session import (
     RemoteExecutionClientSession,
     get_session,
 )
-from exponent.core.remote_execution.tool_execution import execute_tool, truncate_result
+from exponent.core.remote_execution.tool_execution import (
+    execute_bash_tool,
+    execute_tool,
+    truncate_result,
+)
 from exponent.core.remote_execution.types import (
     ChatSource,
     CLIConnectedState,
-    CodeExecutionRequest,
     CreateChatResponse,
     GitInfo,
-    HaltRequest,
-    HaltResponse,
     HeartbeatInfo,
     RemoteExecutionResponseType,
     RunWorkflowRequest,
     StreamingCodeExecutionRequest,
-    SwitchCLIChatRequest,
-    SwitchCLIChatResponse,
 )
 from exponent.core.remote_execution.utils import (
     deserialize_api_response,
-    serialize_message,
 )
 from exponent.utils.version import get_installed_version
 
@@ -138,43 +142,27 @@ class RemoteExecutionClient:
         data = json.dumps(msg_data["data"])
         request = msgspec.json.decode(data, type=CliRpcRequest)
 
-        if isinstance(request, HaltRequest):
-            logger.debug(
-                f"Received HaltRequest for correlation_id {request.correlation_id}"
-            )
+        if isinstance(request.request, TerminateRequest):
             await self.halt_all_code_executions()
-            data = json.loads(
-                serialize_message(
-                    HaltResponse(
-                        correlation_id=request.correlation_id,
-                    )
+            await websocket.send(
+                json.dumps(
+                    {
+                        "type": "result",
+                        "data": msgspec.to_builtins(
+                            CliRpcResponse(
+                                request_id=request.request_id,
+                                response=TerminateResponse(),
+                            )
+                        ),
+                    }
                 )
             )
-            msg = json.dumps({"type": "result", "data": data})
-            logger.debug(
-                f"Sending HaltResponse for correlation_id {request.correlation_id}"
-            )
-            await websocket.send(msg)
-            return None
-
-        elif isinstance(request, SwitchCLIChatRequest):
-            data = json.loads(
-                serialize_message(
-                    SwitchCLIChatResponse(
-                        correlation_id=request.correlation_id,
-                    )
-                )
-            )
-            msg = json.dumps({"type": "result", "data": data})
-            await websocket.send(msg)
-            return SwitchCLIChat(new_chat_uuid=request.new_chat_uuid)
-
-        elif isinstance(request, (CodeExecutionRequest, StreamingCodeExecutionRequest)):
-            await self.add_code_execution_to_halt_states(request.correlation_id)
-            await requests.put(request)
-            return None
-
         else:
+            if isinstance(request.request, ToolExecutionRequest) and isinstance(
+                request.request.tool_input, BashToolInput
+            ):
+                await self.add_code_execution_to_halt_states(request.request_id)
+
             await requests.put(request)
             return None
 
@@ -419,9 +407,18 @@ class RemoteExecutionClient:
     async def handle_request(self, request: CliRpcRequest) -> CliRpcResponse:
         try:
             if isinstance(request.request, ToolExecutionRequest):
-                tool_result = await execute_tool(
-                    request.request.tool_input, self.working_directory
-                )
+                if isinstance(request.request.tool_input, BashToolInput):
+                    tool_result = await execute_bash_tool(
+                        request.request.tool_input,
+                        self.working_directory,
+                        should_halt=self.get_halt_check(
+                            request.request.tool_input.command
+                        ),
+                    )
+                else:
+                    tool_result = await execute_tool(
+                        request.request.tool_input, self.working_directory
+                    )
                 tool_result = truncate_result(tool_result)
                 return CliRpcResponse(
                     request_id=request.request_id,
@@ -429,6 +426,16 @@ class RemoteExecutionClient:
                         tool_result=tool_result,
                     ),
                 )
+            elif isinstance(request.request, GetAllFilesRequest):
+                files = await file_walk(self.working_directory)
+                return CliRpcResponse(
+                    request_id=request.request_id,
+                    response=GetAllFilesResponse(files=files),
+                )
+            elif isinstance(request.request, TerminateRequest):
+                raise ValueError(
+                    "TerminateRequest should not be handled by handle_request"
+                )
 
             raise ValueError(f"Unhandled request type: {type(request)}")
 
@@ -437,10 +444,10 @@ class RemoteExecutionClient:
             raise e
         finally:
             # Clean up halt state after request is complete
-            if isinstance(request, CodeExecutionRequest) or isinstance(
-                request, StreamingCodeExecutionRequest
+            if isinstance(request.request, ToolExecutionRequest) and isinstance(
+                request.request.tool_input, BashToolInput
             ):
-                await self.clear_halt_state(request.correlation_id)
+                await self.clear_halt_state(request.request_id)
 
     async def handle_streaming_request(
         self, request: StreamingCodeExecutionRequest
